@@ -11,11 +11,7 @@ impl Generator {
             for param in &fun.inputs {
                 params.push(param.typ_name.clone());
             }
-            variants.push(format!(
-                "{}({})",
-                fun.name,
-                &itertools::join(params, ","),
-            ));
+            variants.push(format!("{}({})", fun.name, &itertools::join(params, ","),));
         }
         format!(
             "
@@ -30,11 +26,7 @@ impl Generator {
     fn generate_response(&self, svc: &Service) -> String {
         let mut variants = vec![];
         for fun in &svc.functions {
-            variants.push(format!(
-                "{}({})",
-                fun.name,
-                fun.output,
-            ));
+            variants.push(format!("{}({})", fun.name, fun.output,));
         }
         format!(
             "
@@ -60,9 +52,8 @@ impl Generator {
     fn generate_server_struct(&self, svc: &Service) -> String {
         format!(
             "
-	#[derive(Clone)]
 	pub struct {svc_name}Service<App: {svc_name}> {{
-		app: App
+		app: std::sync::Arc<App>
 	}}
 	",
             svc_name = svc.name,
@@ -71,32 +62,27 @@ impl Generator {
     fn generate_trait(&self, svc: &Service) -> String {
         let mut methods = vec![];
         for fun in &svc.functions {
-            let mut params = vec!["self".to_owned()];
+            let mut params = vec!["&self".to_owned()];
             for param in &fun.inputs {
-                params.push(format!(
-                    "{}:{}",
-                    param.var_name,
-                    param.typ_name,
-                ));
+                params.push(format!("{}:{}", param.var_name, param.typ_name,));
             }
             let params = itertools::join(params, ",");
             methods.push(format!(
                 "async fn {}({}) -> {};",
-                fun.name,
-                &params,
-                fun.output,
+                fun.name, &params, fun.output,
             ));
         }
         format!(
             "
-		#[norpc::async_trait{no_send}]
-		pub trait {svc_name}: Clone {{
+		#[norpc::async_trait{no_send_marker}]
+		pub trait {svc_name}: Sync {no_send_constraint} {{
 			{}
 		}}
 		",
             itertools::join(methods, ""),
             svc_name = svc.name,
-            no_send = if self.no_send { "(?Send)" } else { "" },
+            no_send_marker = if self.no_send { "(?Send)" } else { "" },
+            no_send_constraint = if self.no_send { "" } else { "+ Send " },
         )
     }
     fn generate_client_impl(&self, svc: &Service) -> String {
@@ -104,11 +90,7 @@ impl Generator {
         for fun in &svc.functions {
             let mut params = vec!["&mut self".to_owned()];
             for p in &fun.inputs {
-                params.push(format!(
-                    "{}:{}",
-                    p.var_name,
-                    p.typ_name,
-                ));
+                params.push(format!("{}:{}", p.var_name, p.typ_name,));
             }
             let params = itertools::join(params, ",");
 
@@ -120,11 +102,11 @@ impl Generator {
 
             let f = format!(
                 "
-		pub async fn {fun_name}({params}) -> std::result::Result<{output}, Svc::Error> {{
+		pub async fn {fun_name}({params}) -> {output} {{
             norpc::poll_fn(|ctx| self.svc.poll_ready(ctx)).await.ok();
-			let rep = self.svc.call({}Request::{fun_name}({req_params})).await?;
+			let rep = self.svc.call({}Request::{fun_name}({req_params})).await;
 			match rep {{
-				{svc_name}Response::{fun_name}(v) => Ok(v),
+				Ok({svc_name}Response::{fun_name}(v)) => v,
                 #[allow(unreachable_patterns)]
 				_ => unreachable!(),
 			}}
@@ -179,7 +161,7 @@ impl Generator {
         "
 	impl<App: {svc_name}> {svc_name}Service<App> {{
 		pub fn new(app: App) -> Self {{
-			Self {{ app }}
+			Self {{ app: std::sync::Arc::new(app) }}
 		}}
 	}}
     impl<App: {svc_name} + 'static {no_send}> norpc::Service<{svc_name}Request> for {svc_name}Service<App> {{
